package cnv.hf.widgets;

import java.util.ArrayList;
import java.util.List;

import hmi.mapctrls.HPMapView;
import hmi.packages.HPDefine.HPLRect;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.drawable.BitmapDrawable;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.VelocityTracker;
import android.view.View;
import android.view.ViewConfiguration;
import android.view.ViewGroup;
import android.widget.Scroller;
import cnv.hf.widgets.HFModeActivity.HFModeActivityState;

public class HFSlideMenu extends ViewGroup {
	public static Class<?> m_MenuCls = null;
	public static int mMenuWidth = 0;
	public static final int SCREEN_MENU = 0;
	public static final int SCREEN_MAIN = 1;
	private static final int SCREEN_INVALID = -1;
	
	public static class SlideMenuOrientation{
	   public static final int	ORIENTATION_LTR		= 0;
	   public static final int	ORIENTATION_RTL		= 1;
	}
	
	private int mCurrentScreen;
	private int mNextScreen = SCREEN_INVALID;
    
	private Scroller mScroller;
	private VelocityTracker mVelocityTracker;
	private int mTouchSlop;
	
	private float mLastMotionX;
	private float mLastMotionY;

	private final static int TOUCH_STATE_REST = 0;
	private final static int TOUCH_STATE_SCROLLING = 1;
	private static final int SNAP_VELOCITY = 1000;

	public int mTouchState = TOUCH_STATE_REST;
	private boolean mLocked;
	private boolean mAllowLongPress;
	private boolean mAllowDraging = false;
	private View mMenu = null;
	private View mContent = null;
	private int mSlideOrientation = SlideMenuOrientation.ORIENTATION_RTL;
	private List<HFIOnSlideMenuChangedListener> mOnSlideMenuChangedListeners;
	public int getSlideOrientation(){
		return mSlideOrientation;
	}
	
	public void setSlideDirection(int orientation){
		mSlideOrientation = orientation;
	}
	
	public HFSlideMenu(Context context) {
		this(context, null, 0);
	}

	public HFSlideMenu(Context context, AttributeSet attrs) {
		this(context, attrs, 0);
	}

	public HFSlideMenu(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);

		mScroller = new Scroller(getContext());
		mCurrentScreen = SCREEN_MAIN;
		mTouchSlop = ViewConfiguration.get(getContext()).getScaledTouchSlop();
	}

	@Override
	protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
		super.onMeasure(widthMeasureSpec, heightMeasureSpec);
		measureViews(widthMeasureSpec, heightMeasureSpec);
	}

	public void measureViews(int widthMeasureSpec, int heightMeasureSpec) {
		View menuView = getChildAt(0);
		menuView.measure(menuView.getLayoutParams().width + menuView.getLeft()
				+ menuView.getRight(), heightMeasureSpec);

		View contentView = getChildAt(1);
		contentView.measure(widthMeasureSpec, heightMeasureSpec);
	}

	@Override
	protected void onLayout(boolean changed, int l, int t, int r, int b) {
		int childCount = getChildCount();
		if (childCount != 2) {
			throw new IllegalStateException(
					"The childCount of SlidingMenu must be 2");
		}

		View menuView = getChildAt(0);
		View contentView = getChildAt(1);
		if(mSlideOrientation == SlideMenuOrientation.ORIENTATION_LTR)
		{
		    final  int width = menuView.getMeasuredWidth();
		    menuView.layout(-width, 0, 0, menuView.getMeasuredHeight());
		}else{
			final  int width = menuView.getMeasuredWidth();
			final  int left = contentView.getMeasuredWidth();
		    menuView.layout(left, 0, left + width, menuView.getMeasuredHeight());
		}

	
		contentView.layout(0, 0, contentView.getMeasuredWidth(),
				contentView.getMeasuredHeight());
		
		//ָıص
		if(mOnSlideMenuChangedListeners != null && changed){
			for (HFIOnSlideMenuChangedListener listener : mOnSlideMenuChangedListeners) {
				if(listener != null)
				    listener.changed();
			}
		}
	}
	
	@Override
	protected void onFinishInflate() {
		super.onFinishInflate();
		View child;
		for (int i = 0; i < getChildCount(); i++) {
			child = getChildAt(i);
			child.setFocusable(true);
			child.setClickable(true);
		}
	}
	
	@Override
	public boolean onInterceptTouchEvent(MotionEvent ev) {
		if (mLocked) {
			return true;
		}
		mAllowDraging = false;
		if(mCurrentScreen == SCREEN_MENU)
			mAllowDraging = true;
		if(!isAllowDraging()) {
			return false;
		}
		final int action = ev.getAction();
		if ((action == MotionEvent.ACTION_MOVE)
				&& (mTouchState != TOUCH_STATE_REST)) {
			return true;
		}

		final float x = ev.getX();
		final float y = ev.getY();

		switch (action) {
		case MotionEvent.ACTION_MOVE:

			final int xDiff = (int) Math.abs(x - mLastMotionX);
			final int yDiff = (int) Math.abs(y - mLastMotionY);

			final int touchSlop = mTouchSlop;
			boolean xMoved = xDiff > touchSlop;
			boolean yMoved = yDiff > touchSlop;

			if (xMoved || yMoved) {

				if (xMoved) {
					// Scroll if the user moved far enough along the X axis
					mTouchState = TOUCH_STATE_SCROLLING;
					enableChildrenCache();
				}
				// Either way, cancel any pending longpress
				if (mAllowLongPress) {
					mAllowLongPress = false;
					// Try canceling the long press. It could also have been
					// scheduled
					// by a distant descendant, so use the mAllowLongPress flag
					// to block
					// everything
					final View currentScreen = getChildAt(mCurrentScreen);
					currentScreen.cancelLongPress();
				}
			}
			break;

		case MotionEvent.ACTION_DOWN:
			// Remember location of down touch
			mLastMotionX = x;
			mLastMotionY = y;
			mAllowLongPress = true;

			mTouchState = mScroller.isFinished() ? TOUCH_STATE_REST
					: TOUCH_STATE_SCROLLING;

			break;

		case MotionEvent.ACTION_CANCEL:
		case MotionEvent.ACTION_UP:
			// Release the drag
			clearChildrenCache();
			mTouchState = TOUCH_STATE_REST;
			mAllowLongPress = false;
			break;
		}

		/*
		 * The only time we want to intercept motion events is if we are in the
		 * drag mode.
		 */
		return mTouchState != TOUCH_STATE_REST;
	}
	private boolean enableCache = false;
	void enableChildrenCache() {
		BitmapDrawable drawable = null;
		enableCache = true;
		HFModeWidget modeWidget = HFModesManager.getCurrentMode();
		HFModeFragment mode = null;
		if(modeWidget != null && modeWidget instanceof HFModeFragment)
			mode = (HFModeFragment)modeWidget;
		if(mode != null && mode.getModeActivityState() == HFModeActivityState.ActivityState_onOnResume){
		   HFMapWidget mapWidget = mode.getMapWidget();
		   if(mapWidget != null){
			 Bitmap bitmap = null;
			 HPMapView mapView= mapWidget.getMapView();
			 HPLRect rect = new HPLRect();
			 rect.left = (0);
			 rect.top = (0);
			 rect.right = (HFModesManager.getScreenWidth());
			 rect.bottom = (HFModesManager.getScreenHeight()-HFModesManager.getStatusBarHeight());
			 bitmap = mapView.getMapScreenShot(rect);  
			 if(bitmap != null)
			    drawable = new BitmapDrawable(bitmap);
			 if(drawable != null && drawable.getBitmap().isRecycled() == false){
		       mapWidget.getObject().setBackgroundDrawable(drawable);
			 }
		   }
		}
		final int count = getChildCount();
		for (int i = 0; i < count; i++) {
			final View layout = (View) getChildAt(i);
			if(i == 0){//˵ͼ
			    layout.setDrawingCacheEnabled(true);
			}
			else{	
				if(drawable != null && drawable.getBitmap().isRecycled() == false && mode != null && mode.getModeActivityState() == HFModeActivityState.ActivityState_onOnResume){
			    layout.setDrawingCacheEnabled(true);
				}
			}
		}
	}

	void clearChildrenCache() {
		HFModeWidget modeWidget = HFModesManager.getCurrentMode();
		HFModeFragment mode = null;
		if(modeWidget != null && modeWidget instanceof HFModeFragment)
			mode = (HFModeFragment)modeWidget;
		enableCache = false;
		if(mode != null)/*&& mode.getModeActivityState() == HFModeActivityState.ActivityState_onOnResume)*/{
		   HFMapWidget mapWidget = mode.getMapWidget();
		   if(mapWidget != null){
			 HPMapView mapView= mapWidget.getMapView();
		     mapWidget.getObject().setBackgroundDrawable(null);
		     mapView.recycleMapBitmap();
		   }
		}
		final int count = getChildCount();
		for (int i = 0; i < count; i++) {
			final View layout = (View) getChildAt(i);
			if(i == 0){//˵ͼ
			    layout.setDrawingCacheEnabled(false);
			}
			else{
				layout.setDrawingCacheEnabled(false);
			}
		}
	}
	
    void clearMenuCache(){
		if(enableCache == true && mScroller.isFinished()){
			mCurrentScreen = SCREEN_MAIN;
			clearChildrenCache();
			abortAnimation = false;
			mNextScreen = SCREEN_INVALID;
		}
	}
	
	@Override
	public boolean onTouchEvent(MotionEvent ev) {
		if (mLocked) {
			return true;
		}
		if(!isAllowDraging()) {
			return false;
		}
		if (mVelocityTracker == null) {
			mVelocityTracker = VelocityTracker.obtain();
		}
		mVelocityTracker.addMovement(ev);

		final int action = ev.getAction();
		final float x = ev.getX();

		switch (action) {
		case MotionEvent.ACTION_DOWN:
			/*
			 * If being flinged and user touches, stop the fling. isFinished
			 * will be false if being flinged.
			 */
			if (!mScroller.isFinished()) {
				abortAnimation = true;
				mScroller.abortAnimation();
			}

			// Remember where the motion event started
			mLastMotionX = x;
			break;
		case MotionEvent.ACTION_MOVE:
			if (mTouchState == TOUCH_STATE_SCROLLING) {
				// Scroll to follow the motion event
				int deltaX = (int) (mLastMotionX - x);
				mLastMotionX = x;

				if(mSlideOrientation == SlideMenuOrientation.ORIENTATION_LTR){
					if (deltaX < 0) {
						if (deltaX + getScrollX() >= -getChildAt(0).getWidth()) {
							scrollBy(deltaX, 0);
						}
	
					} else if (deltaX > 0) {
						final int availableToScroll = getChildAt(
								getChildCount() - 1).getRight()
								- getScrollX() - getWidth();
	
						if (availableToScroll > 0) {
							scrollBy(Math.min(availableToScroll, deltaX), 0);
						}
					}
				}else if(mSlideOrientation == SlideMenuOrientation.ORIENTATION_RTL){
					if (deltaX > 0) {
						if (deltaX + getScrollX() <= getChildAt(0).getWidth()) {
							scrollBy(deltaX, 0);
						}
	
					} else if (deltaX < 0) {
						final int availableToScroll = getChildAt(
								getChildCount() - 1).getRight()
								+ getScrollX() - getWidth();
	
						if (availableToScroll > 0) {
							scrollBy(Math.max(-availableToScroll, deltaX), 0);
						}
					}
				}
			}
			break;
		case MotionEvent.ACTION_UP:
			if (mTouchState == TOUCH_STATE_SCROLLING) {
				final VelocityTracker velocityTracker = mVelocityTracker;
				velocityTracker.computeCurrentVelocity(1000);
				int velocityX = (int) velocityTracker.getXVelocity();
                if(mSlideOrientation == SlideMenuOrientation.ORIENTATION_RTL)
                	velocityX = -velocityX;
				if (velocityX > SNAP_VELOCITY && mCurrentScreen == SCREEN_MAIN) {
					// Fling hard enough to move left
					snapToScreen(SCREEN_MENU,-1);
				} else if (velocityX < -SNAP_VELOCITY
						&& mCurrentScreen == SCREEN_MENU) {
					// Fling hard enough to move right
					snapToScreen(SCREEN_MAIN,-1);
				} else {
					snapToDestination();
				}

				if (mVelocityTracker != null) {
					mVelocityTracker.recycle();
					mVelocityTracker = null;
				}
			}
			mTouchState = TOUCH_STATE_REST;
			break;
		case MotionEvent.ACTION_CANCEL:
			mTouchState = TOUCH_STATE_REST;
		}

		return true;
	}
	private boolean abortAnimation = false;
	@Override
	public void computeScroll() {
		if (mScroller.computeScrollOffset()) {
			scrollTo(mScroller.getCurrX(), mScroller.getCurrY());
		} else if (mNextScreen != SCREEN_INVALID) {
			mCurrentScreen = Math.max(0,
					Math.min(mNextScreen, getChildCount() - 1));
			mNextScreen = SCREEN_INVALID;
			if(!abortAnimation)
			   clearChildrenCache();
			abortAnimation = false;
		}
	}

	@Override
	public void scrollTo(int x, int y) {
		super.scrollTo(x, y);
		postInvalidate();
	}

	@Override
	protected void dispatchDraw(Canvas canvas) {
		final int scrollX = getScrollX();
		super.dispatchDraw(canvas);
		canvas.translate(scrollX, 0);
	}

	@Override
	public boolean dispatchUnhandledMove(View focused, int direction) {
		if (direction == View.FOCUS_LEFT) {
			if (getCurrentScreen() > 0) {
				snapToScreen(getCurrentScreen() - 1,-1);
				return true;
			}
		} else if (direction == View.FOCUS_RIGHT) {
			if (getCurrentScreen() < getChildCount() - 1) {
				snapToScreen(getCurrentScreen() + 1,-1);
				return true;
			}
		}
		return super.dispatchUnhandledMove(focused, direction);
	}
	
	protected void snapToScreen(int whichScreen,int milliTime) {
		int duration = milliTime;
		enableChildrenCache();

		whichScreen = Math.max(0, Math.min(whichScreen, getChildCount() - 1));
		boolean changingScreens = whichScreen != mCurrentScreen;

		mNextScreen = whichScreen;

		View focusedChild = getFocusedChild();
		if (focusedChild != null && changingScreens
				&& focusedChild == getChildAt(mCurrentScreen)) {
			focusedChild.clearFocus();
		}
		switch(mSlideOrientation){
		case SlideMenuOrientation.ORIENTATION_LTR:
		{
			final int newX = (whichScreen - 1) * getChildAt(0).getWidth();
			final int delta = newX - getScrollX();
			if(duration < 0)
				duration = Math.abs(delta) * 2;
			if(duration > 800)
				duration = 800;
			if(duration > 0)//0ֱӹɵĿλ
			    mScroller.startScroll(getScrollX(), 0, delta, 0, duration);
			else
				scrollTo(getScrollX()+delta, 0); 
			break;
		}
		case SlideMenuOrientation.ORIENTATION_RTL:
		{
			final int newX = (1 - whichScreen) * getChildAt(0).getWidth();
			final int delta = newX - getScrollX();
			if(duration < 0)
				duration = Math.abs(delta) * 2;
			if(duration > 800)
				duration = 800;
		    if(duration > 0)//0ֱӹɵĿλ
			   mScroller.startScroll(getScrollX(), 0, delta, 0, duration);
		    else
		       scrollTo(getScrollX()+delta, 0);
			break;
		}
		default:
			break;
		}
		invalidate();
	}
	protected void snapToDestination() {
		if (getScrollX() == 0) {
			clearMenuCache();
			return;
		}
		final int screenWidth = getChildAt(0).getWidth();
		int whichScreen = (screenWidth + getScrollX() + (screenWidth / 2))/ screenWidth;
		if(mSlideOrientation == SlideMenuOrientation.ORIENTATION_RTL)
			whichScreen = (screenWidth - getScrollX() + (screenWidth / 2))/ screenWidth;
		snapToScreen(whichScreen,-1);
	}
	
	public int getCurrentScreen() {
		return mCurrentScreen;
	}
	
	public boolean isMainScreenShowing() {
		return mCurrentScreen == SCREEN_MAIN;
	}
	
	public void openMenu() {
		mCurrentScreen = SCREEN_MENU;
		snapToScreen(mCurrentScreen,-1);
	}
	
	public void openMenu(int milliTime){
		mCurrentScreen = SCREEN_MENU;
		snapToScreen(mCurrentScreen,milliTime);
	}
	
	public void closeMenu() {
		mCurrentScreen = SCREEN_MAIN;
		snapToScreen(mCurrentScreen,-1);
	}
	
	public void closeMenu(int milliTime) {
		mCurrentScreen = SCREEN_MAIN;
		snapToScreen(mCurrentScreen,milliTime);
	}
	
	public void resetLayout(){
		if(mCurrentScreen == SCREEN_MAIN)
			return;
		mCurrentScreen = SCREEN_MAIN;
		int whichScreen = mCurrentScreen;
		whichScreen = Math.max(0, Math.min(whichScreen, getChildCount() - 1));
		boolean changingScreens = whichScreen != mCurrentScreen;

		mNextScreen = whichScreen;

		View focusedChild = getFocusedChild();
		if (focusedChild != null && changingScreens
				&& focusedChild == getChildAt(mCurrentScreen)) {
			focusedChild.clearFocus();
		}
		switch(mSlideOrientation){
		case SlideMenuOrientation.ORIENTATION_LTR:
		{
			final int newX = (whichScreen - 1) * getChildAt(0).getWidth();
			final int delta = newX - getScrollX();
			mScroller.startScroll(getScrollX(), 0, delta, 0, 0);
			break;
		}
		case SlideMenuOrientation.ORIENTATION_RTL:
		{
			final int newX = (1 - whichScreen) * getChildAt(0).getWidth();
			final int delta = newX - getScrollX();
			mScroller.startScroll(getScrollX(), 0, delta, 0, 0);
			break;
		}
		default:
			break;
		}
		invalidate();
	}
	
	public void unlock() {
		mLocked = false;
	}
	public void lock() {
		mLocked = true;
	}
	public boolean isAllowDraging() {
		return mAllowDraging;
	}
	public void setAllowDraging(boolean allowDraging) {
		mAllowDraging = allowDraging;
	}
	public View getMenu() {
		return mMenu;
	}
	public void setMenu(View menu) {
		if(mMenu != null) {
			removeView(mMenu);
		}
		if(menu != null) {
			addView(menu, SCREEN_MENU);
		}
		mMenu = menu;
	}
	public void setMenu(Class<?> cls,int mOrientation){
		HFModeFragment fragment = null;
		m_MenuCls = cls;
		try {
			fragment = (HFModeFragment) cls.newInstance();
		} catch (Exception e) {
			e.printStackTrace();
		}
		if(fragment != null) {
			HFModeActivity mMainMode = (HFModeActivity) this.getContext();
			FragmentManager fm = mMainMode.getSupportFragmentManager();
			FragmentTransaction ft = fm.beginTransaction();	
			ft.replace(HFModeActivity.HF_MENU_CONTENT_ID, fragment);
			ft.commitAllowingStateLoss();
			mSlideOrientation = mOrientation;
		}
	}
	public View getContent() {
		return mContent;
	}
	public void setContent(View content) {
		if(mContent != null) {
			removeView(mContent);
		}		
		if(content != null) {
			addView(content);
		}		
		mContent = content;
	}
	public void setContent(View content,LayoutParams layoutParams) {
		if(mContent != null) {
			removeView(mContent);
		}		
		if(content != null) {
			addView(content,layoutParams);
		}		
		mContent = content;
	}
	protected void changeOrientation(int orientation){
		if(mCurrentScreen == SCREEN_MAIN)
			return;
		switch(mSlideOrientation){
		case SlideMenuOrientation.ORIENTATION_LTR:
		{
			scrollTo(mMenuWidth, 0);
			break;
		}
		case SlideMenuOrientation.ORIENTATION_RTL:
		{
			scrollTo(mMenuWidth, 0);
			break;
		}
		default:
			break;
		}
		invalidate();
	}
	
	/** üSlideMenuָı¼
	 * @param listener
	 * @return void
	 * @author LiangYJ
	 * @date 2015-9-4 10:13:28
	 */ 
	public void addOnSlideMenuChangedListener(HFIOnSlideMenuChangedListener listener){
		if(mOnSlideMenuChangedListeners == null){
			mOnSlideMenuChangedListeners = new ArrayList<HFSlideMenu.HFIOnSlideMenuChangedListener>();
			mOnSlideMenuChangedListeners.add(listener);
		}
	}
	/** 
	 * SlideMenuıӿ
	 * @author LiangYJ
	 * @date 2015-9-4 10:19:40
	 */
	public interface HFIOnSlideMenuChangedListener {
		public void changed();
	}
}
